@@ -3,8 +3,7 @@
 /*               This file is part of the program and library                */
 /*    PaPILO --- Parallel Presolve for Integer and Linear Optimization       */
 /*                                                                           */
-/* Copyright (C) 2020-2023 Konrad-Zuse-Zentrum                               */
-/*                     fuer Informationstechnik Berlin                       */
+/* Copyright (C) 2020-2024 Zuse Institute Berlin (ZIB)                       */
 /*                                                                           */
 /* This program is free software: you can redistribute it and/or modify      */
 /* it under the terms of the GNU Lesser General Public License as published  */
@@ -311,9 +310,9 @@ class SparseStorage
       if( i != rowranges[row].end )
       {
          indbuffer.insert( indbuffer.end(), &columns[i],
-                           &columns[rowranges[row].end] );
+                           &columns.data()[rowranges[row].end] );
          valbuffer.insert( valbuffer.end(), &values[i],
-                           &values[rowranges[row].end] );
+                           &values.data()[rowranges[row].end] );
       }
       else
       {
