@@ -3,8 +3,7 @@
 /*               This file is part of the program and library                */
 /*    PaPILO --- Parallel Presolve for Integer and Linear Optimization       */
 /*                                                                           */
-/* Copyright (C) 2020-2023 Konrad-Zuse-Zentrum                               */
-/*                     fuer Informationstechnik Berlin                       */
+/* Copyright (C) 2020-2024 Zuse Institute Berlin (ZIB)                       */
 /*                                                                           */
 /* This program is free software: you can redistribute it and/or modify      */
 /* it under the terms of the GNU Lesser General Public License as published  */
@@ -97,7 +96,8 @@ struct Validation
 
       return parser.read(
           optimal_solution_file, one_to_one_mapping,
-          postsolveStorage.getOriginalProblem().getVariableNames(), optimal_solution );
+          postsolveStorage.getOriginalProblem().getVariableNames(),
+          optimal_solution.primal );
    }
 
    static bool
