@@ -3,8 +3,7 @@
 /*               This file is part of the program and library                */
 /*    PaPILO --- Parallel Presolve for Integer and Linear Optimization       */
 /*                                                                           */
-/* Copyright (C) 2020-2022 Konrad-Zuse-Zentrum                               */
-/*                     fuer Informationstechnik Berlin                       */
+/* Copyright (C) 2020-2024 Zuse Institute Berlin (ZIB)                       */
 /*                                                                           */
 /* This program is free software: you can redistribute it and/or modify      */
 /* it under the terms of the GNU Lesser General Public License as published  */
@@ -40,13 +39,15 @@ class ConstraintPropagation : public PresolveMethod<REAL>
    {
       this->setName( "propagation" );
       this->setTiming( PresolverTiming::kFast );
+      this->setArgument(ArgumentType::kPropagation);
    }
 
    /// todo how to communicate about postsolve information
    PresolveStatus
    execute( const Problem<REAL>& problem,
-            const ProblemUpdate<REAL>& problemUpdate, const Num<REAL>& num,
-            Reductions<REAL>& reductions, const Timer& timer ) override;
+            const ProblemUpdate<REAL>& problemUpdate,
+            const Num<REAL>& num, Reductions<REAL>& reductions,
+            const Timer& timer, int& reason_of_infeasibility) override;
 };
 
 #ifdef PAPILO_USE_EXTERN_TEMPLATES
@@ -59,9 +60,8 @@ template <typename REAL>
 PresolveStatus
 ConstraintPropagation<REAL>::execute( const Problem<REAL>& problem,
                                       const ProblemUpdate<REAL>& problemUpdate,
-                                      const Num<REAL>& num,
-                                      Reductions<REAL>& reductions, const Timer& timer )
-{
+                                      const Num<REAL>& num, Reductions<REAL>& reductions,
+                                      const Timer& timer, int& reason_of_infeasibility){
    const auto& domains = problem.getVariableDomains();
    const auto& activities = problem.getRowActivities();
    const auto& changedactivities = problemUpdate.getChangedActivities();
