@@ -3,8 +3,7 @@
 /*               This file is part of the program and library                */
 /*    PaPILO --- Parallel Presolve for Integer and Linear Optimization       */
 /*                                                                           */
-/* Copyright (C) 2020-2023 Konrad-Zuse-Zentrum                               */
-/*                     fuer Informationstechnik Berlin                       */
+/* Copyright (C) 2020-2024 Zuse Institute Berlin (ZIB)                       */
 /*                                                                           */
 /* This program is free software: you can redistribute it and/or modify      */
 /* it under the terms of the GNU Lesser General Public License as published  */
@@ -44,6 +43,8 @@ class DominatedCols : public PresolveMethod<REAL>
    DominatedCols() : PresolveMethod<REAL>()
    {
       this->setName( "domcol" );
+      /// argument case can be primal or dual depending on the sign of the domination
+      this->setArgument( ArgumentType::kPrimal );
       this->setTiming( PresolverTiming::kExhaustive );
    }
 
@@ -99,7 +100,8 @@ class DominatedCols : public PresolveMethod<REAL>
    PresolveStatus
    execute( const Problem<REAL>& problem,
             const ProblemUpdate<REAL>& problemUpdate, const Num<REAL>& num,
-            Reductions<REAL>& reductions, const Timer& timer ) override;
+            Reductions<REAL>& reductions, const Timer& timer,
+            int& reason_of_infeasibility ) override;
 };
 
 #ifdef PAPILO_USE_EXTERN_TEMPLATES
@@ -112,9 +114,8 @@ template <typename REAL>
 PresolveStatus
 DominatedCols<REAL>::execute( const Problem<REAL>& problem,
                               const ProblemUpdate<REAL>& problemUpdate,
-                              const Num<REAL>& num,
-                              Reductions<REAL>& reductions, const Timer& timer )
-{
+                              const Num<REAL>& num, Reductions<REAL>& reductions,
+                              const Timer& timer, int& reason_of_infeasibility){
    const auto& obj = problem.getObjective().coefficients;
    const auto& consMatrix = problem.getConstraintMatrix();
    const auto& lbValues = problem.getLowerBounds();
@@ -360,13 +361,13 @@ DominatedCols<REAL>::execute( const Problem<REAL>& problem,
    for( int k = 0; k < (int) unboundedcols.size(); ++k )
 #endif
           {
-             int i = unboundedcols[k];
-             int lbfree = colinfo[i].lbfree;
-             int ubfree = colinfo[i].ubfree;
+             int unbounded_col = unboundedcols[k];
+             int lbfree = colinfo[unbounded_col].lbfree;
+             int ubfree = colinfo[unbounded_col].ubfree;
 
              assert( lbfree != 0 || ubfree != 0 );
 
-             auto colvec = consMatrix.getColumnCoefficients( i );
+             auto colvec = consMatrix.getColumnCoefficients( unbounded_col );
              int collen = colvec.getLength();
              const int* colrows = colvec.getIndices();
              const REAL* colvals = colvec.getValues();
@@ -412,8 +413,8 @@ DominatedCols<REAL>::execute( const Problem<REAL>& problem,
 
              auto candrowvec =
                  consMatrix.getRowCoefficients( colrows[bestrow] );
-             REAL colval = colvals[bestrow] * scale;
-             REAL colobj = obj[i] * scale;
+             REAL scaled_val = colvals[bestrow] * scale;
+             REAL scaled_obj = obj[unbounded_col] * scale;
              bestrow = colrows[bestrow];
              int rowlen = candrowvec.getLength();
              const int* rowcols = candrowvec.getIndices();
@@ -422,66 +423,66 @@ DominatedCols<REAL>::execute( const Problem<REAL>& problem,
              for( int j = 0; j != rowlen; ++j )
              {
                 int col = rowcols[j];
-                if( col == i || ( cflags[i].test( ColFlag::kIntegral ) &&
+                if( col == unbounded_col || ( cflags[unbounded_col].test( ColFlag::kIntegral ) &&
                                   !cflags[col].test( ColFlag::kIntegral ) ) )
                    continue;
 
-                bool fixtolb = false;
-                bool fixtoub = false;
+                bool to_lb = false;
+                bool to_ub = false;
 
                 if( !rflags[bestrow].test( RowFlag::kLhsInf,
                                            RowFlag::kRhsInf ) )
                 {
                    if( !cflags[col].test( ColFlag::kLbInf ) &&
-                       num.isEq( colval, rowvals[j] ) &&
-                       num.isLE( colobj, obj[col] ) &&
-                       checkDominance( i, col, scale, 1 ) )
-                      fixtolb = true;
+                       num.isEq( scaled_val, rowvals[j] ) &&
+                       num.isLE( scaled_obj, obj[col] ) &&
+                       checkDominance( unbounded_col, col, scale, 1 ) )
+                      to_lb = true;
 
                    if( !cflags[col].test( ColFlag::kUbInf ) &&
-                       num.isEq( colval, -rowvals[j] ) &&
-                       num.isLE( colobj, -obj[col] ) &&
-                       checkDominance( i, col, scale, -1 ) )
-                      fixtoub = true;
+                       num.isEq( scaled_val, -rowvals[j] ) &&
+                       num.isLE( scaled_obj, -obj[col] ) &&
+                       checkDominance( unbounded_col, col, scale, -1 ) )
+                      to_ub = true;
                 }
                 else if( rflags[bestrow].test( RowFlag::kLhsInf ) )
                 {
-                   assert( colval > 0 &&
+                   assert( scaled_val > 0 &&
                            !rflags[bestrow].test( RowFlag::kRhsInf ) );
                    if( !cflags[col].test( ColFlag::kLbInf ) &&
-                       num.isLE( colval, rowvals[j] ) &&
-                       num.isLE( colobj, obj[col] ) &&
-                       checkDominance( i, col, scale, 1 ) )
-                      fixtolb = true;
+                       num.isLE( scaled_val, rowvals[j] ) &&
+                       num.isLE( scaled_obj, obj[col] ) &&
+                       checkDominance( unbounded_col, col, scale, 1 ) )
+                      to_lb = true;
 
                    if( !cflags[col].test( ColFlag::kUbInf ) &&
-                       num.isLE( colval, -rowvals[j] ) &&
-                       num.isLE( colobj, -obj[col] ) &&
-                       checkDominance( i, col, scale, -1 ) )
-                      fixtoub = true;
+                       num.isLE( scaled_val, -rowvals[j] ) &&
+                       num.isLE( scaled_obj, -obj[col] ) &&
+                       checkDominance( unbounded_col, col, scale, -1 ) )
+                      to_ub = true;
                 }
                 else
                 {
-                   assert( colval < 0 &&
+                   assert( scaled_val < 0 &&
                            rflags[bestrow].test( RowFlag::kRhsInf ) );
                    if( !cflags[col].test( ColFlag::kLbInf ) &&
-                       num.isGE( colval, rowvals[j] ) &&
-                       num.isLE( colobj, obj[col] ) &&
-                       checkDominance( i, col, scale, 1 ) )
-                      fixtolb = true;
+                       num.isGE( scaled_val, rowvals[j] ) &&
+                       num.isLE( scaled_obj, obj[col] ) &&
+                       checkDominance( unbounded_col, col, scale, 1 ) )
+                      to_lb = true;
 
                    if( !cflags[col].test( ColFlag::kUbInf ) &&
-                       num.isGE( colval, -rowvals[j] ) &&
-                       num.isLE( colobj, -obj[col] ) &&
-                       checkDominance( i, col, scale, -1 ) )
-                      fixtoub = true;
+                       num.isGE( scaled_val, -rowvals[j] ) &&
+                       num.isLE( scaled_obj, -obj[col] ) &&
+                       checkDominance( unbounded_col, col, scale, -1 ) )
+                      to_ub = true;
                 }
 
-                if( fixtolb || fixtoub )
+                if( to_lb || to_ub )
                 {
                    domcolreductions.push_back( DomcolReduction{
-                       i, col, implrowlock,
-                       fixtolb ? BoundChange::kUpper : BoundChange::kLower } );
+                       unbounded_col, col, implrowlock,
+                       to_lb ? BoundChange::kUpper : BoundChange::kLower } );
                 }
              }
           }
@@ -525,14 +526,21 @@ DominatedCols<REAL>::execute( const Problem<REAL>& problem,
          reductions.lockColBounds( dr.col1 );
          reductions.lockCol( dr.col2 );
          reductions.lockColBounds( dr.col2 );
-         //TODO: check if >=0 is correct instead of >0
          if( dr.implrowlock >= 0 )
             reductions.lockRow( dr.implrowlock );
 
+         // upper bound is changed to lower bound
          if( dr.boundchg == BoundChange::kUpper )
+         {
+            reductions.dominance(dr.col2, dr.col1);
             reductions.fixCol( dr.col2, lbValues[dr.col2], dr.implrowlock );
+         }
+         // lower bound is changed to upper bound
          else
-            reductions.fixCol( dr.col2, ubValues[dr.col2], dr.implrowlock);
+         {
+            reductions.dominance(dr.col1, dr.col2);
+            reductions.fixCol( dr.col2, ubValues[dr.col2], dr.implrowlock );
+         }
       }
 
    }
@@ -540,6 +548,7 @@ DominatedCols<REAL>::execute( const Problem<REAL>& problem,
    return result;
 }
 
+
 } // namespace papilo
 
 #endif
