@@ -3,8 +3,7 @@
 /*               This file is part of the program and library                */
 /*    PaPILO --- Parallel Presolve for Integer and Linear Optimization       */
 /*                                                                           */
-/* Copyright (C) 2020-2022 Konrad-Zuse-Zentrum                               */
-/*                     fuer Informationstechnik Berlin                       */
+/* Copyright (C) 2020-2024 Zuse Institute Berlin (ZIB)                       */
 /*                                                                           */
 /* This program is free software: you can redistribute it and/or modify      */
 /* it under the terms of the GNU Lesser General Public License as published  */
@@ -41,6 +40,7 @@ class DualFix : public PresolveMethod<REAL>
    DualFix() : PresolveMethod<REAL>()
    {
       this->setName( "dualfix" );
+      this->setArgument( ArgumentType::kDual );
       this->setTiming( PresolverTiming::kMedium );
    }
 
@@ -70,8 +70,10 @@ class DualFix : public PresolveMethod<REAL>
 
    PresolveStatus
    execute( const Problem<REAL>& problem,
-            const ProblemUpdate<REAL>& problemUpdate, const Num<REAL>& num,
-            Reductions<REAL>& reductions, const Timer& timer ) override;
+            const ProblemUpdate<REAL>& problemUpdate,
+            const Num<REAL>& num, Reductions<REAL>& reductions,
+            const Timer& timer, int& reason_of_infeasibility)
+       override;
 
  private:
    PresolveStatus
@@ -97,8 +99,8 @@ template <typename REAL>
 PresolveStatus
 DualFix<REAL>::execute( const Problem<REAL>& problem,
                         const ProblemUpdate<REAL>& problemUpdate,
-                        const Num<REAL>& num, Reductions<REAL>& reductions, const Timer& timer )
-{
+                        const Num<REAL>& num, Reductions<REAL>& reductions,
+                        const Timer& timer, int& reason_of_infeasibility){
    const auto& consMatrix = problem.getConstraintMatrix();
    const Vec<RowActivity<REAL>>& activities = problem.getRowActivities();
    const Vec<ColFlags>& cflags = problem.getColFlags();
