@@ -3,8 +3,7 @@
 /*               This file is part of the program and library                */
 /*    PaPILO --- Parallel Presolve for Integer and Linear Optimization       */
 /*                                                                           */
-/* Copyright (C) 2020-2023 Konrad-Zuse-Zentrum                               */
-/*                     fuer Informationstechnik Berlin                       */
+/* Copyright (C) 2020-2024 Zuse Institute Berlin (ZIB)                       */
 /*                                                                           */
 /* This program is free software: you can redistribute it and/or modify      */
 /* it under the terms of the GNU Lesser General Public License as published  */
@@ -51,11 +50,36 @@ class DualInfer : public PresolveMethod<REAL>
          this->setEnabled( false );
       return false;
    }
+   
+   bool
+   is_primal_problem_bounded( const Problem<REAL>& problem, const Num<REAL>& num, int& primal_bounded )
+   {
+      assert( primal_bounded == 1 || primal_bounded == 0 || primal_bounded == -1 );
+      if(primal_bounded == 0)
+           return false;
+      if(primal_bounded == 1)
+           return true;
+      auto& colflags = problem.getColFlags();
+      auto& coefficients = problem.getObjective().coefficients;
+      for( int i = 0; i < problem.getNCols(); i++ )
+      {
+           if( ( num.isGT( coefficients[i], 0 ) && colflags[i].test( ColFlag::kLbInf ) )
+               || ( num.isLT( coefficients[i], 0 ) && colflags[i].test( ColFlag::kUbInf ) ) )
+           {
+            primal_bounded = 0;
+            return false;
+         }
+      }
+      primal_bounded = 1;
+      return true;
+   }
 
    virtual PresolveStatus
    execute( const Problem<REAL>& problem,
-            const ProblemUpdate<REAL>& problemUpdate, const Num<REAL>& num,
-            Reductions<REAL>& reductions, const Timer& timer ) override;
+            const ProblemUpdate<REAL>& problemUpdate,
+            const Num<REAL>& num, Reductions<REAL>& reductions,
+            const Timer& timer, int& reason_of_infeasibility)
+       override;
 };
 
 #ifdef PAPILO_USE_EXTERN_TEMPLATES
@@ -68,8 +92,8 @@ template <typename REAL>
 PresolveStatus
 DualInfer<REAL>::execute( const Problem<REAL>& problem,
                           const ProblemUpdate<REAL>& problemUpdate,
-                          const Num<REAL>& num, Reductions<REAL>& reductions, const Timer& timer )
-{
+                          const Num<REAL>& num, Reductions<REAL>& reductions,
+                          const Timer& timer, int& reason_of_infeasibility){
    assert( problem.getNumContinuousCols() != 0 );
 
    const auto& obj = problem.getObjective().coefficients;
@@ -321,7 +345,7 @@ DualInfer<REAL>::execute( const Problem<REAL>& problem,
          }
 
          // reject too small bound changes
-         if( !oldboundinf && newbound - oldbound <= +1000 * num.getFeasTol() )
+         if( !oldboundinf && num.isFeasLE((newbound - oldbound) / (1000 * num.max(abs(oldbound), 1)), 0) )
             return;
 
          dualColFlags[dualCol].unset( ColFlag::kLbInf );
@@ -364,7 +388,7 @@ DualInfer<REAL>::execute( const Problem<REAL>& problem,
          }
 
          // reject too small bound changes
-         if( !oldboundinf && newbound - oldbound >= -1000 * num.getFeasTol() )
+         if( !oldboundinf && num.isFeasGE((newbound - oldbound) / (1000 * num.max(abs(oldbound), 1)), 0) )
             return;
 
          dualColFlags[dualCol].unset( ColFlag::kUbInf );
@@ -448,6 +472,8 @@ DualInfer<REAL>::execute( const Problem<REAL>& problem,
    int fixedints = 0;
    int fixedconts = 0;
 
+   int primal_bounded = -1; // -1 not initialized
+
    // change inequality rows to equations if their dual value cannot be zero
    for( int i = 0; i < nrows; ++i )
    {
@@ -463,6 +489,12 @@ DualInfer<REAL>::execute( const Problem<REAL>& problem,
       {
          assert( !rflags[i].test( RowFlag::kLhsInf ) );
 
+         if( dualLB[i] == std::numeric_limits<REAL>::infinity() && !num.isZero( lhsValues[i] ) )
+            return PresolveStatus::kUnbndOrInfeas;
+
+         if( !is_primal_problem_bounded( problem, num, primal_bounded ) )
+            continue;
+         
          if( activities[i].ninfmax != 0 ||
              num.isFeasLT( lhsValues[i], activities[i].max ) )
          {
@@ -477,6 +509,12 @@ DualInfer<REAL>::execute( const Problem<REAL>& problem,
       {
          assert( !rflags[i].test( RowFlag::kRhsInf ) );
 
+         if( abs(dualUB[i]) == std::numeric_limits<REAL>::infinity() && !num.isZero( rhsValues[i] ) )
+            return PresolveStatus::kUnbndOrInfeas;
+
+         if( !is_primal_problem_bounded( problem, num, primal_bounded ) )
+            continue;
+
          if( activities[i].ninfmin != 0 ||
              num.isFeasGT( rhsValues[i], activities[i].min ) )
          {
