@@ -3,8 +3,7 @@
 /*               This file is part of the program and library                */
 /*    PaPILO --- Parallel Presolve for Integer and Linear Optimization       */
 /*                                                                           */
-/* Copyright (C) 2020-2023 Konrad-Zuse-Zentrum                               */
-/*                     fuer Informationstechnik Berlin                       */
+/* Copyright (C) 2020-2024 Zuse Institute Berlin (ZIB)                       */
 /*                                                                           */
 /* This program is free software: you can redistribute it and/or modify      */
 /* it under the terms of the GNU Lesser General Public License as published  */
@@ -55,8 +54,9 @@ class SimpleSubstitution : public PresolveMethod<REAL>
 
    PresolveStatus
    execute( const Problem<REAL>& problem,
-            const ProblemUpdate<REAL>& problemUpdate, const Num<REAL>& num,
-            Reductions<REAL>& reductions, const Timer& timer ) override;
+            const ProblemUpdate<REAL>& problemUpdate,
+            const Num<REAL>& num, Reductions<REAL>& reductions,
+            const Timer& timer, int& reason_of_infeasibility) override;
 
 
 
@@ -87,9 +87,8 @@ template <typename REAL>
 PresolveStatus
 SimpleSubstitution<REAL>::execute( const Problem<REAL>& problem,
                                    const ProblemUpdate<REAL>& problemUpdate,
-                                   const Num<REAL>& num,
-                                   Reductions<REAL>& reductions, const Timer& timer )
-{
+                                   const Num<REAL>& num, Reductions<REAL>& reductions,
+                                   const Timer& timer, int& reason_of_infeasibility){
    // go over the rows and get the equalities, extract the columns that
    // verify the conditions add them to a hash map, loop over the hash map
    // and compute the implied bounds and finally look for implied free
